"""fes http://www.uif.shcp.gob.mx/recepcion/fes"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class AlertaType(ScalarMap):
    """
    
    :param tipo_alerta:
    :param descripcion_alerta:
    """
    
    def __init__(
            self,
            tipo_alerta: str,
            descripcion_alerta: str = None,
    ): 
        super().__init__({
            'TipoAlerta': tipo_alerta,
            'DescripcionAlerta': descripcion_alerta,
        })
        

class TribunalType(ScalarMap):
    """
    
    :param clave_tribunal_dependencia:
    :param clave_actividad:
    """
    
    def __init__(
            self,
            clave_tribunal_dependencia: str,
            clave_actividad: str,
    ): 
        super().__init__({
            'ClaveTribunalDependencia': clave_tribunal_dependencia,
            'ClaveActividad': clave_actividad,
        })
        

class PropietarioType(ScalarMap):
    """
    
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class DatosPropietarioType(ScalarMap):
    """
    
    :param propietario_solicita:
    :param dato_propietario:
    """
    
    def __init__(
            self,
            propietario_solicita: str,
            dato_propietario: PropietarioType | dict | Sequence[PropietarioType | dict] = None,
    ): 
        super().__init__({
            'PropietarioSolicita': propietario_solicita,
            'DatoPropietario': dato_propietario,
        })
        

class DatosLiquidacionType(ScalarMap):
    """
    
    :param moneda:
    :param monto_operacion:
    """
    
    def __init__(
            self,
            moneda: str,
            monto_operacion: str,
    ): 
        super().__init__({
            'Moneda': moneda,
            'MontoOperacion': monto_operacion,
        })
        

class DatosGarantiaType(ScalarMap):
    """
    
    :param tipo_garantia:
    :param datos_bien_garantia:
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_garantia: str,
            datos_bien_garantia: BienGarantiaType | dict = None,
            tipo_persona: TipoGaranteType | dict = None,
    ): 
        super().__init__({
            'TipoGarantia': tipo_garantia,
            'DatosBienGarantia': datos_bien_garantia,
            'TipoPersona': tipo_persona,
        })
        

class BienGarantiaType(ScalarMap):
    """
    
    :param datos_inmueble:
    :param datos_otro:
    """
    
    def __init__(
            self,
            datos_inmueble: GarantiaInmuebleType | dict = None,
            datos_otro: GarantiaOtroType | dict = None,
    ): 
        super().__init__({
            'DatosInmueble': datos_inmueble,
            'DatosOtro': datos_otro,
        })
        

class GarantiaOtroType(ScalarMap):
    """
    
    :param descripcion_garantia:
    """
    
    def __init__(
            self,
            descripcion_garantia: str,
    ): 
        super().__init__({
            'DescripcionGarantia': descripcion_garantia,
        })
        

class GarantiaInmuebleType(ScalarMap):
    """
    
    :param tipo_inmueble:
    :param valor_referencia:
    :param codigo_postal:
    :param folio_real:
    """
    
    def __init__(
            self,
            tipo_inmueble: str,
            valor_referencia: str,
            codigo_postal: str,
            folio_real: str,
    ): 
        super().__init__({
            'TipoInmueble': tipo_inmueble,
            'ValorReferencia': valor_referencia,
            'CodigoPostal': codigo_postal,
            'FolioReal': folio_real,
        })
        

class TipoGaranteType(ScalarMap):
    """
    
    :param persona_fisica:
    :param persona_moral:
    :param fideicomiso:
    """
    
    def __init__(
            self,
            persona_fisica: GaranteFisicaType | dict = None,
            persona_moral: GaranteMoralType | dict = None,
            fideicomiso: GaranteFideicomisoType | dict = None,
    ): 
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class GaranteFideicomisoType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param rfc:
    :param identificador_fideicomiso:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class GaranteMoralType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param fecha_constitucion:
    :param rfc:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class GaranteFisicaType(ScalarMap):
    """
    
    :param nombre:
    :param apellido_paterno:
    :param apellido_materno:
    :param fecha_nacimiento:
    :param rfc:
    :param curp:
    """
    
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class DatosDeudorType(ScalarMap):
    """
    
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class DatosAcreedorType(ScalarMap):
    """
    
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class DatosModificacionType(ScalarMap):
    """
    
    :param tipo_modificacion_capital_fijo:
    :param inicial_capital_fijo:
    :param final_capital_fijo:
    :param tipo_modificacion_capital_variable:
    :param inicial_capital_variable:
    :param final_capital_variable:
    :param datos_accionista:
    """
    
    def __init__(
            self,
            tipo_modificacion_capital_fijo: str,
            inicial_capital_fijo: str,
            final_capital_fijo: str,
            tipo_modificacion_capital_variable: str,
            inicial_capital_variable: str,
            final_capital_variable: str,
            datos_accionista: DatosAccionista1Type | dict | Sequence[DatosAccionista1Type | dict],
    ): 
        super().__init__({
            'TipoModificacionCapitalFijo': tipo_modificacion_capital_fijo,
            'InicialCapitalFijo': inicial_capital_fijo,
            'FinalCapitalFijo': final_capital_fijo,
            'TipoModificacionCapitalVariable': tipo_modificacion_capital_variable,
            'InicialCapitalVariable': inicial_capital_variable,
            'FinalCapitalVariable': final_capital_variable,
            'DatosAccionista': datos_accionista,
        })
        

class PersonaMoralModificaType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param pais_nacionalidad:
    :param giro_mercantil:
    :param numero_total_acciones:
    :param motivo_modificacion:
    :param fecha_constitucion:
    :param rfc:
    :param instrumento_publico:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            giro_mercantil: str,
            numero_total_acciones: str,
            motivo_modificacion: str,
            fecha_constitucion: date = None,
            rfc: str = None,
            instrumento_publico: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'GiroMercantil': giro_mercantil,
            'NumeroTotalAcciones': numero_total_acciones,
            'MotivoModificacion': motivo_modificacion,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
            'InstrumentoPublico': instrumento_publico,
        })
        

class DatosAccionista1Type(ScalarMap):
    """
    
    :param tipo_persona:
    :param numero_acciones:
    """
    
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
            numero_acciones: str,
    ): 
        super().__init__({
            'TipoPersona': tipo_persona,
            'NumeroAcciones': numero_acciones,
        })
        

class CapitalSocialType(ScalarMap):
    """
    
    :param capital_fijo:
    :param capital_variable:
    """
    
    def __init__(
            self,
            capital_fijo: str,
            capital_variable: str = None,
    ): 
        super().__init__({
            'CapitalFijo': capital_fijo,
            'CapitalVariable': capital_variable,
        })
        

class DatosAccionistaType(ScalarMap):
    """
    
    :param cargo_accionista:
    :param tipo_persona:
    :param numero_acciones:
    """
    
    def __init__(
            self,
            cargo_accionista: str,
            tipo_persona: TipoPersonaSimpleType | dict,
            numero_acciones: str,
    ): 
        super().__init__({
            'CargoAccionista': cargo_accionista,
            'TipoPersona': tipo_persona,
            'NumeroAcciones': numero_acciones,
        })
        

class PersonaMoralConstType(ScalarMap):
    """
    
    :param tipo_persona_moral:
    :param denominacion_razon:
    :param giro_mercantil:
    :param numero_total_acciones:
    :param consejo_vigilancia:
    :param motivo_constitucion:
    :param datos_accionista:
    :param capital_social:
    :param tipo_persona_moral_otra:
    :param folio_mercantil:
    :param entidad_federativa:
    :param instrumento_publico:
    """
    
    def __init__(
            self,
            tipo_persona_moral: str,
            denominacion_razon: str,
            giro_mercantil: str,
            numero_total_acciones: str,
            consejo_vigilancia: str,
            motivo_constitucion: str,
            datos_accionista: DatosAccionistaType | dict | Sequence[DatosAccionistaType | dict],
            capital_social: CapitalSocialType | dict,
            tipo_persona_moral_otra: str = None,
            folio_mercantil: str = None,
            entidad_federativa: str = None,
            instrumento_publico: str = None,
    ): 
        super().__init__({
            'TipoPersonaMoral': tipo_persona_moral,
            'DenominacionRazon': denominacion_razon,
            'GiroMercantil': giro_mercantil,
            'NumeroTotalAcciones': numero_total_acciones,
            'ConsejoVigilancia': consejo_vigilancia,
            'MotivoConstitucion': motivo_constitucion,
            'DatosAccionista': datos_accionista,
            'CapitalSocial': capital_social,
            'TipoPersonaMoralOtra': tipo_persona_moral_otra,
            'FolioMercantil': folio_mercantil,
            'EntidadFederativa': entidad_federativa,
            'InstrumentoPublico': instrumento_publico,
        })
        

class Autoridad1Type(ScalarMap):
    """
    
    :param tipo_autoridad:
    :param domicilio_oficina:
    """
    
    def __init__(
            self,
            tipo_autoridad: TipoAutoridad1Type | dict,
            domicilio_oficina: DomicilioOficinaType | dict = None,
    ): 
        super().__init__({
            'TipoAutoridad': tipo_autoridad,
            'DomicilioOficina': domicilio_oficina,
        })
        

class TipoAutoridad1Type(ScalarMap):
    """
    
    :param administrativo:
    :param jurisdiccional:
    """
    
    def __init__(
            self,
            administrativo: Administrativo1Type | dict = None,
            jurisdiccional: JurisdiccionalType | dict = None,
    ): 
        super().__init__({
            'Administrativo': administrativo,
            'Jurisdiccional': jurisdiccional,
        })
        

class DatosApoderadoType(ScalarMap):
    """
    
    :param tipo_poder:
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_poder: str,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        super().__init__({
            'TipoPoder': tipo_poder,
            'TipoPersona': tipo_persona,
        })
        

class DatosPoderdanteType(ScalarMap):
    """
    
    :param tipo_persona:
    """
    
    def __init__(
            self,
            tipo_persona: TipoPersonaSimpleType | dict,
    ): 
        super().__init__({
            'TipoPersona': tipo_persona,
        })
        

class TipoPersonaSimpleType(ScalarMap):
    """
    
    :param persona_fisica:
    :param persona_moral:
    :param fideicomiso:
    """
    
    def __init__(
            self,
            persona_fisica: PersonaFisicaSimpleType | dict = None,
            persona_moral: PersonaMoralSimpleType | dict = None,
            fideicomiso: FideicomisoSimpleType | dict = None,
    ): 
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class AutoridadType(ScalarMap):
    """
    
    :param tipo_autoridad:
    :param domicilio_oficina:
    """
    
    def __init__(
            self,
            tipo_autoridad: TipoAutoridadType | dict,
            domicilio_oficina: DomicilioOficinaType | dict = None,
    ): 
        super().__init__({
            'TipoAutoridad': tipo_autoridad,
            'DomicilioOficina': domicilio_oficina,
        })
        

class DomicilioOficinaType(ScalarMap):
    """
    
    :param nacional:
    :param extranjero:
    """
    
    def __init__(
            self,
            nacional: NacionalType | dict = None,
            extranjero: ExtranjeroType | dict = None,
    ): 
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class TipoAutoridadType(ScalarMap):
    """
    
    :param administrativo:
    :param jurisdiccional:
    """
    
    def __init__(
            self,
            administrativo: AdministrativoType | dict = None,
            jurisdiccional: JurisdiccionalType | dict = None,
    ): 
        super().__init__({
            'Administrativo': administrativo,
            'Jurisdiccional': jurisdiccional,
        })
        

class JurisdiccionalType(ScalarMap):
    """
    
    :param organo:
    :param tipo_juicio:
    :param materia:
    :param expediente:
    """
    
    def __init__(
            self,
            organo: str,
            tipo_juicio: str,
            materia: str,
            expediente: str,
    ): 
        super().__init__({
            'Organo': organo,
            'TipoJuicio': tipo_juicio,
            'Materia': materia,
            'Expediente': expediente,
        })
        

class AdministrativoType(ScalarMap):
    """
    
    :param organo:
    :param cargo:
    :param instrumento_publico:
    """
    
    def __init__(
            self,
            organo: str,
            cargo: str,
            instrumento_publico: str,
    ): 
        super().__init__({
            'Organo': organo,
            'Cargo': cargo,
            'InstrumentoPublico': instrumento_publico,
        })
        

class TelefonoType(ScalarMap):
    """
    
    :param numero_telefono:
    :param clave_pais:
    :param correo_electronico:
    """
    
    def __init__(
            self,
            numero_telefono: str,
            clave_pais: str = None,
            correo_electronico: str = None,
    ): 
        super().__init__({
            'NumeroTelefono': numero_telefono,
            'ClavePais': clave_pais,
            'CorreoElectronico': correo_electronico,
        })
        

class ExtranjeroType(ScalarMap):
    """
    
    :param pais:
    :param estado_provincia:
    :param ciudad_poblacion:
    :param colonia:
    :param calle:
    :param numero_exterior:
    :param codigo_postal:
    :param numero_interior:
    """
    
    def __init__(
            self,
            pais: str,
            estado_provincia: str,
            ciudad_poblacion: str,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        super().__init__({
            'Pais': pais,
            'EstadoProvincia': estado_provincia,
            'CiudadPoblacion': ciudad_poblacion,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class NacionalType(ScalarMap):
    """
    
    :param colonia:
    :param calle:
    :param numero_exterior:
    :param codigo_postal:
    :param numero_interior:
    """
    
    def __init__(
            self,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            numero_interior: str = None,
    ): 
        super().__init__({
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'NumeroInterior': numero_interior,
        })
        

class TipoDomicilioType(ScalarMap):
    """
    
    :param nacional:
    :param extranjero:
    """
    
    def __init__(
            self,
            nacional: NacionalType | dict = None,
            extranjero: ExtranjeroType | dict = None,
    ): 
        super().__init__({
            'Nacional': nacional,
            'Extranjero': extranjero,
        })
        

class DatosPersonaActoType(ScalarMap):
    """
    
    :param caracter:
    :param tipo_persona:
    :param caracter_otro:
    :param tipo_domicilio:
    :param telefono:
    """
    
    def __init__(
            self,
            caracter: str,
            tipo_persona: TipoPersonaType | dict,
            caracter_otro: str = None,
            tipo_domicilio: TipoDomicilioType | dict = None,
            telefono: TelefonoType | dict = None,
    ): 
        super().__init__({
            'Caracter': caracter,
            'TipoPersona': tipo_persona,
            'CaracterOtro': caracter_otro,
            'TipoDomicilio': tipo_domicilio,
            'Telefono': telefono,
        })
        

class FideicomisoType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param apoderado_delegado:
    :param rfc:
    :param identificador_fideicomiso:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            apoderado_delegado: RepresentanteApoderadoType | dict,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'ApoderadoDelegado': apoderado_delegado,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaMoralType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param pais_nacionalidad:
    :param representante_apoderado:
    :param fecha_constitucion:
    :param rfc:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            representante_apoderado: RepresentanteApoderadoType | dict,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'RepresentanteApoderado': representante_apoderado,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class PersonaFisicaType(ScalarMap):
    """
    
    :param nombre:
    :param apellido_paterno:
    :param apellido_materno:
    :param pais_nacionalidad:
    :param fecha_nacimiento:
    :param rfc:
    :param curp:
    """
    
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            pais_nacionalidad: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'PaisNacionalidad': pais_nacionalidad,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class TipoPersonaType(ScalarMap):
    """
    
    :param persona_fisica:
    :param persona_moral:
    :param fideicomiso:
    """
    
    def __init__(
            self,
            persona_fisica: PersonaFisicaType | dict = None,
            persona_moral: PersonaMoralType | dict = None,
            fideicomiso: FideicomisoType | dict = None,
    ): 
        super().__init__({
            'PersonaFisica': persona_fisica,
            'PersonaMoral': persona_moral,
            'Fideicomiso': fideicomiso,
        })
        

class PersonasActoType(ScalarMap):
    """
    
    :param datos_persona_acto:
    """
    
    def __init__(
            self,
            datos_persona_acto: DatosPersonaActoType | dict | Sequence[DatosPersonaActoType | dict],
    ): 
        super().__init__({
            'DatosPersonaActo': datos_persona_acto,
        })
        

class InmuebleType(ScalarMap):
    """
    
    :param tipo_inmueble:
    :param valor_catastral:
    :param colonia:
    :param calle:
    :param numero_exterior:
    :param codigo_postal:
    :param dimension_terreno:
    :param dimension_construido:
    :param folio_real:
    :param numero_interior:
    """
    
    def __init__(
            self,
            tipo_inmueble: str,
            valor_catastral: str,
            colonia: str,
            calle: str,
            numero_exterior: str,
            codigo_postal: str,
            dimension_terreno: str,
            dimension_construido: str,
            folio_real: str,
            numero_interior: str = None,
    ): 
        super().__init__({
            'TipoInmueble': tipo_inmueble,
            'ValorCatastral': valor_catastral,
            'Colonia': colonia,
            'Calle': calle,
            'NumeroExterior': numero_exterior,
            'CodigoPostal': codigo_postal,
            'DimensionTerreno': dimension_terreno,
            'DimensionConstruido': dimension_construido,
            'FolioReal': folio_real,
            'NumeroInterior': numero_interior,
        })
        

class DatosInmueblesType(ScalarMap):
    """
    
    :param caracteristicas_inmueble:
    """
    
    def __init__(
            self,
            caracteristicas_inmueble: InmuebleType | dict | Sequence[InmuebleType | dict],
    ): 
        super().__init__({
            'CaracteristicasInmueble': caracteristicas_inmueble,
        })
        

class AvaluoType(ScalarMap):
    """
    
    :param organo:
    :param cargo:
    :param expediente_oficio:
    :param persona_solicita:
    :param tipo_bien:
    :param valor_avaluo:
    :param datos_propietario:
    :param descripcion:
    """
    
    def __init__(
            self,
            organo: str,
            cargo: str,
            expediente_oficio: str,
            persona_solicita: PersonaFisicaType | dict,
            tipo_bien: str,
            valor_avaluo: str,
            datos_propietario: DatosPropietarioType | dict,
            descripcion: str = None,
    ): 
        super().__init__({
            'Organo': organo,
            'Cargo': cargo,
            'ExpedienteOficio': expediente_oficio,
            'PersonaSolicita': persona_solicita,
            'TipoBien': tipo_bien,
            'ValorAvaluo': valor_avaluo,
            'DatosPropietario': datos_propietario,
            'Descripcion': descripcion,
        })
        

class ContratoMutuoType(ScalarMap):
    """
    
    :param autoridad:
    :param tipo_otorgamiento:
    :param persona_solicita:
    :param datos_acreedor:
    :param datos_deudor:
    :param datos_liquidacion:
    :param datos_garantia:
    """
    
    def __init__(
            self,
            autoridad: AutoridadType | dict,
            tipo_otorgamiento: str,
            persona_solicita: PersonaFisicaSimpleType | dict,
            datos_acreedor: DatosAcreedorType | dict | Sequence[DatosAcreedorType | dict],
            datos_deudor: DatosDeudorType | dict | Sequence[DatosDeudorType | dict],
            datos_liquidacion: DatosLiquidacionType | dict | Sequence[DatosLiquidacionType | dict],
            datos_garantia: DatosGarantiaType | dict | Sequence[DatosGarantiaType | dict] = None,
    ): 
        super().__init__({
            'Autoridad': autoridad,
            'TipoOtorgamiento': tipo_otorgamiento,
            'PersonaSolicita': persona_solicita,
            'DatosAcreedor': datos_acreedor,
            'DatosDeudor': datos_deudor,
            'DatosLiquidacion': datos_liquidacion,
            'DatosGarantia': datos_garantia,
        })
        

class ModificacionPatrimonialType(ScalarMap):
    """
    
    :param autoridad:
    :param persona_moral_modifica:
    :param datos_modificacion:
    """
    
    def __init__(
            self,
            autoridad: Autoridad1Type | dict,
            persona_moral_modifica: PersonaMoralModificaType | dict,
            datos_modificacion: DatosModificacionType | dict,
    ): 
        super().__init__({
            'Autoridad': autoridad,
            'PersonaMoralModifica': persona_moral_modifica,
            'DatosModificacion': datos_modificacion,
        })
        

class ConstitucionPmType(ScalarMap):
    """
    
    :param autoridad:
    :param persona_solicita:
    :param persona_moral_constitucion:
    """
    
    def __init__(
            self,
            autoridad: Autoridad1Type | dict,
            persona_solicita: PersonaFisicaType | dict,
            persona_moral_constitucion: PersonaMoralConstType | dict,
    ): 
        super().__init__({
            'Autoridad': autoridad,
            'PersonaSolicita': persona_solicita,
            'PersonaMoralConstitucion': persona_moral_constitucion,
        })
        

class OtorgamientoPoderType(ScalarMap):
    """
    
    :param autoridad:
    :param persona_solicita:
    :param datos_poderdante:
    :param datos_apoderado:
    """
    
    def __init__(
            self,
            autoridad: AutoridadType | dict,
            persona_solicita: PersonaFisicaType | dict,
            datos_poderdante: DatosPoderdanteType | dict | Sequence[DatosPoderdanteType | dict],
            datos_apoderado: DatosApoderadoType | dict | Sequence[DatosApoderadoType | dict],
    ): 
        super().__init__({
            'Autoridad': autoridad,
            'PersonaSolicita': persona_solicita,
            'DatosPoderdante': datos_poderdante,
            'DatosApoderado': datos_apoderado,
        })
        

class DerechosInmuebleType(ScalarMap):
    """
    
    :param organo:
    :param tipo_juicio:
    :param materia:
    :param expediente:
    :param tipo_acto:
    :param datos_inmuebles:
    :param personas_acto:
    :param tipo_acto_otro:
    """
    
    def __init__(
            self,
            organo: str,
            tipo_juicio: str,
            materia: str,
            expediente: str,
            tipo_acto: str,
            datos_inmuebles: DatosInmueblesType | dict,
            personas_acto: PersonasActoType | dict,
            tipo_acto_otro: str = None,
    ): 
        super().__init__({
            'Organo': organo,
            'TipoJuicio': tipo_juicio,
            'Materia': materia,
            'Expediente': expediente,
            'TipoActo': tipo_acto,
            'DatosInmuebles': datos_inmuebles,
            'PersonasActo': personas_acto,
            'TipoActoOtro': tipo_acto_otro,
        })
        

class ActividadType(ScalarMap):
    """
    
    :param derechos_inmuebles:
    :param otorgamiento_poder:
    :param constitucion_personas_morales:
    :param modificacion_patrimonial:
    :param contrato_mutuo_credito:
    :param avaluo:
    """
    
    def __init__(
            self,
            derechos_inmuebles: DerechosInmuebleType | dict = None,
            otorgamiento_poder: OtorgamientoPoderType | dict = None,
            constitucion_personas_morales: ConstitucionPmType | dict = None,
            modificacion_patrimonial: ModificacionPatrimonialType | dict = None,
            contrato_mutuo_credito: ContratoMutuoType | dict = None,
            avaluo: AvaluoType | dict = None,
    ): 
        super().__init__({
            'DerechosInmuebles': derechos_inmuebles,
            'OtorgamientoPoder': otorgamiento_poder,
            'ConstitucionPersonasMorales': constitucion_personas_morales,
            'ModificacionPatrimonial': modificacion_patrimonial,
            'ContratoMutuoCredito': contrato_mutuo_credito,
            'Avaluo': avaluo,
        })
        

class AvisoType(ScalarMap):
    """
    
    :param referencia_aviso:
    :param prioridad:
    :param alerta:
    :param detalle_operaciones:
    :param modificatorio:
    """
    
    def __init__(
            self,
            referencia_aviso: str,
            prioridad: str,
            alerta: AlertaType | dict,
            detalle_operaciones: OperacionesType | dict,
            modificatorio: ModificatorioType | dict = None,
    ): 
        super().__init__({
            'ReferenciaAviso': referencia_aviso,
            'Prioridad': prioridad,
            'Alerta': alerta,
            'DetalleOperaciones': detalle_operaciones,
            'Modificatorio': modificatorio,
        })
        

class ModificatorioType(ScalarMap):
    """
    
    :param folio_modificacion:
    :param descripcion_modificacion:
    """
    
    def __init__(
            self,
            folio_modificacion: str,
            descripcion_modificacion: str,
    ): 
        super().__init__({
            'FolioModificacion': folio_modificacion,
            'DescripcionModificacion': descripcion_modificacion,
        })
        

class OperacionType(ScalarMap):
    """
    
    :param fecha_operacion:
    :param tipo_actividad:
    """
    
    def __init__(
            self,
            fecha_operacion: date,
            tipo_actividad: ActividadType | dict,
    ): 
        super().__init__({
            'FechaOperacion': fecha_operacion,
            'TipoActividad': tipo_actividad,
        })
        

class OperacionesType(ScalarMap):
    """
    
    :param datos_operacion:
    """
    
    def __init__(
            self,
            datos_operacion: OperacionType | dict | Sequence[OperacionType | dict],
    ): 
        super().__init__({
            'DatosOperacion': datos_operacion,
        })
        

class ArchivoType(ScalarMap):
    """
    
    :param informe:
    """
    
    def __init__(
            self,
            informe: InformeType | dict | Sequence[InformeType | dict],
    ): 
        super().__init__({
            'Informe': informe,
        })
        

class InformeType(ScalarMap):
    """
    
    :param mes_reportado:
    :param tribunal_dependencia:
    :param aviso:
    :param version:
    """
    
    def __init__(
            self,
            mes_reportado: str,
            tribunal_dependencia: TribunalType | dict,
            aviso: AvisoType | dict | Sequence[AvisoType | dict] = None,
            version: str = None,
    ): 
        super().__init__({
            'MesReportado': mes_reportado,
            'TribunalDependencia': tribunal_dependencia,
            'Aviso': aviso,
            'Version': version,
        })
        

class RepresentanteApoderadoType(ScalarMap):
    """
    
    :param nombre:
    :param apellido_paterno:
    :param apellido_materno:
    :param fecha_nacimiento:
    :param rfc:
    :param curp:
    """
    
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class FideicomisoSimpleType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param rfc:
    :param identificador_fideicomiso:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            rfc: str = None,
            identificador_fideicomiso: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'Rfc': rfc,
            'IdentificadorFideicomiso': identificador_fideicomiso,
        })
        

class PersonaFisicaSimpleType(ScalarMap):
    """
    
    :param nombre:
    :param apellido_paterno:
    :param apellido_materno:
    :param pais_nacionalidad:
    :param fecha_nacimiento:
    :param rfc:
    :param curp:
    """
    
    def __init__(
            self,
            nombre: str,
            apellido_paterno: str,
            apellido_materno: str,
            pais_nacionalidad: str,
            fecha_nacimiento: date = None,
            rfc: str = None,
            curp: str = None,
    ): 
        super().__init__({
            'Nombre': nombre,
            'ApellidoPaterno': apellido_paterno,
            'ApellidoMaterno': apellido_materno,
            'PaisNacionalidad': pais_nacionalidad,
            'FechaNacimiento': fecha_nacimiento,
            'Rfc': rfc,
            'Curp': curp,
        })
        

class PersonaMoralSimpleType(ScalarMap):
    """
    
    :param denominacion_razon:
    :param pais_nacionalidad:
    :param fecha_constitucion:
    :param rfc:
    """
    
    def __init__(
            self,
            denominacion_razon: str,
            pais_nacionalidad: str,
            fecha_constitucion: date = None,
            rfc: str = None,
    ): 
        super().__init__({
            'DenominacionRazon': denominacion_razon,
            'PaisNacionalidad': pais_nacionalidad,
            'FechaConstitucion': fecha_constitucion,
            'Rfc': rfc,
        })
        

class Administrativo1Type(ScalarMap):
    """
    
    :param organo:
    :param cargo:
    :param instrumento_publico_oficio:
    """
    
    def __init__(
            self,
            organo: str,
            cargo: str,
            instrumento_publico_oficio: str,
    ): 
        super().__init__({
            'Organo': organo,
            'Cargo': cargo,
            'InstrumentoPublicoOficio': instrumento_publico_oficio,
        })
        

class Archivo(ArchivoType, XElement):
    tag = '{http://www.uif.shcp.gob.mx/recepcion/fes}archivo'




"""consumodecombustibles http://www.sat.gob.mx/consumodecombustibles"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class Determinado(ScalarMap):
    """
    Nodo para la definición de información detallada de un impuesto específico
    
    :param impuesto: Atributo requerido para definir el tipo de impuesto
    :param tasa: Atributo requerido para señalar la tasa del impuesto por cada concepto amparado en el comprobante
    :param importe: Atributo requerido para definir el importe o monto del impuesto
    """
    
    def __init__(
            self,
            impuesto: str,
            tasa: Decimal | int,
            importe: Decimal | int,
    ): 
        super().__init__({
            'Impuesto': impuesto,
            'Tasa': tasa,
            'Importe': importe,
        })
        

class ConceptoConsumoDeCombustibles(ScalarMap):
    """
    Nodo requerido para la expresión de una transacción para operaciones de compra de combustibles.
    
    :param identificador: Atributo requerido para la expresión del identificador o número del monedero electrónico
    :param fecha: Atributo requerido para la expresión de la Fecha y hora de expedición de la operación reportada. Se expresa en la forma aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO 8601.
    :param rfc: Atributo requerido del RFC del enajenante del combustible
    :param clave_estacion: Atributo requerido para expresar la clave de cliente de la estación de servicio, a 10 caracteres, cuando sea requerido.
    :param cantidad: Atributo requerido para definir el volumen de combustible adquirido.
    :param nombre_combustible: Atributo requerido para expresar el nombre del combustible adquirido.
    :param folio_operacion: Atributo requerido para referir el número de folio de cada operación realizada por cada monedero electrónico.
    :param valor_unitario: Atributo requerido para definir el precio unitario del combustible adquirido.
    :param importe: Atributo requerido para definir el monto total de consumo de combustible. Debe ser equivalente al resultado de multiplicar la cantidad por el valor unitario.
    :param determinados: Nodo requerido para enlistar los impuestos determinados aplicables de combustibles.
    """
    
    def __init__(
            self,
            identificador: str,
            fecha: datetime,
            rfc: str,
            clave_estacion: str,
            cantidad: Decimal | int,
            nombre_combustible: str,
            folio_operacion: str,
            valor_unitario: Decimal | int,
            importe: Decimal | int,
            determinados: Determinado | dict | Sequence[Determinado | dict],
    ): 
        super().__init__({
            'Identificador': identificador,
            'Fecha': fecha,
            'Rfc': rfc,
            'ClaveEstacion': clave_estacion,
            'Cantidad': cantidad,
            'NombreCombustible': nombre_combustible,
            'FolioOperacion': folio_operacion,
            'ValorUnitario': valor_unitario,
            'Importe': importe,
            'Determinados': determinados,
        })
        

class ConsumoDeCombustibles(CFDI):
    """
    Complemento al Comprobante Fiscal Digital por Internet (CFDI) para integrar la información de consumo de combustibles por monedero electrónico.
    
    :param numero_de_cuenta: Nodo requerido para expresar el número de cuenta del adquirente del monedero electrónico
    :param total: Atributo requerido para expresar el monto total de consumo de combustibles.
    :param conceptos: Nodo requerido para enlistar los conceptos cubiertos por Consumo de Combustibles.
    :param sub_total: Atributo opcional para representar la suma de todos los importes tipo ConceptoConsumoDeCombustibles.
    """
    
    tag = '{http://www.sat.gob.mx/consumodecombustibles}ConsumoDeCombustibles'
    version = '1.0'
    tipo_operacion = 'monedero electrónico'
    
    def __init__(
            self,
            numero_de_cuenta: str,
            total: Decimal | int,
            conceptos: ConceptoConsumoDeCombustibles | dict | Sequence[ConceptoConsumoDeCombustibles | dict],
            sub_total: Decimal | int = None,
    ): 
        super().__init__({
            'Version': self.version,
            'TipoOperacion': self.tipo_operacion,
            'NumeroDeCuenta': numero_de_cuenta,
            'Total': total,
            'Conceptos': conceptos,
            'SubTotal': sub_total,
        })
        




from lxml.etree import QName
from weasyprint import HTML, CSS

from .transform import *
from .transform.pdf_environment import PDFEnvironment

PDF_CSS = CSS(string="@page {margin: 1.0cm 1.27cm 1.1cm 0.85cm;}")


class Representable:
    def html_write(self, target, templates_path=None):
        if templates_path:
            env = PDFEnvironment(templates_path=templates_path)
            init_template = env.get_template("_init.html")
        else:
            init_template = PDF_INIT_TEMPLATE

        init_template.stream({"c": self, "k": QName(self.tag).localname}).dump(target)

    def html_str(self, templates_path=None) -> str:
        if templates_path:
            env = PDFEnvironment(templates_path=templates_path)
            init_template = env.get_template("_init.html")
        else:
            init_template = PDF_INIT_TEMPLATE

        return init_template.render({"c": self, "k": QName(self.tag).localname})

    def pdf_write(self, target, templates_path=None):
        HTML(string=self.html_str(templates_path=templates_path)).write_pdf(
            target=target,
            stylesheets=[PDF_CSS]
        )

    def pdf_bytes(self, templates_path=None) -> bytes:
        return HTML(string=self.html_str(templates_path=templates_path)).write_pdf(
            stylesheets=[PDF_CSS]
        )



import os

from satcfdi.cfdi import CFDI
from tests.utils import verify_result

module = 'satcfdi'
current_dir = os.path.dirname(__file__)


def verify_invoice(invoice, path):
    verify = verify_result(
        data=invoice.html_str(
            templates_path=os.path.join(current_dir, 'templates')
        ),
        filename=f"{path}.html"
    )
    assert verify


def test_custom_template():
    xml_file = "cfdv40-ejemplo.xml"
    cfdi = CFDI.from_file(
        os.path.join(current_dir, 'cfdi_ejemplos/comprobante40', xml_file)
    )

    verify_invoice(cfdi, xml_file)



import os

from satcfdi.accounting import SatCFDI

from satcfdi import CFDI

current_dir = os.path.dirname(__file__)


def test_satcfdi():
    file = os.path.join(current_dir, 'cfdi_ejemplos', 'comprobante40/cfdv40-aerolineas.xml')
    c = CFDI.from_file(file)
    d = SatCFDI(c)

    assert d.payments == []
    assert d.relations == []

    assert d["Emisor"]["Rfc"] == "AAA010101AAA"



import base64
from typing import Sequence

import requests
from requests.auth import HTTPBasicAuth

from . import PAC, Accept, Document, CancelReason, CancelationAcknowledgment, Environment
from .. import __version__, ResponseError, Signer
from ..cfdi import CFDI


class Prodigia(PAC):
    """
    PAC de facturacion Prodigia

    Documentacion: https://www.prodigia.com.mx/api-de-integracion-para-timbrado-de-cfdi
    """
    RFC = "PPD101129EA3"

    def __init__(self, user: str, password: str, contrato: str, environment=Environment.PRODUCTION):
        super().__init__(environment=environment)
        self.auth = HTTPBasicAuth(user, password)
        self.contrato = contrato

    def _request(self, path, data, method="post", opciones: Sequence[str] = None):
        host = "https://timbrado.pade.mx/servicio/rest"
        params = [
            ("contrato", self.contrato),
            ("opciones", 'RESPUESTA_JSON'),
            ("opciones", 'REGRESAR_CON_ERROR_307_XML'),
        ]

        if opciones:
            for o in opciones:
                params.append(
                    ('opciones', o)
                )

        r = requests.request(
            method=method,
            url=f"{host}/{path}",
            headers={
                "User-Agent": __version__.__user_agent__,
                "Content-Type": "application/xml",
                # Nota: Header de Accept genera errores
            },
            data=data,
            auth=self.auth,
            params=params
        )

        if r.ok:
            return r
        else:
            raise ResponseError(r)

    def issue(self, cfdi: CFDI, accept: Accept = Accept.XML) -> Document:
        pass

    def stamp(self, cfdi: CFDI, accept: Accept = Accept.XML) -> Document:
        match (self.environment, cfdi.tag):
            case Environment.PRODUCTION, '{http://www.sat.gob.mx/cfd/4}Comprobante':
                path = "timbrado40"
            case Environment.TEST, '{http://www.sat.gob.mx/cfd/4}Comprobante':
                path = "timbrado40Prueba"
            case _:
                raise NotImplementedError("Not Supported CFDI")

        opciones = []
        if accept & Accept.PDF:
            opciones.append("GENERAR_PDF")

        res = self._request(
            path=path,
            data=cfdi.xml_bytes(),
            opciones=opciones
        ).json()

        servicio_timbrado = res['servicioTimbrado']

        if not servicio_timbrado['timbradoOk']:
            raise ResponseError(res)

        return Document(
            document_id=servicio_timbrado['UUID'],
            xml=base64.b64decode(servicio_timbrado['xmlBase64']) if accept & Accept.XML else None,
            pdf=base64.b64decode(servicio_timbrado['pdfBase64']) if accept & Accept.PDF else None
        )

    def cancel(self, cfdi: CFDI, reason: CancelReason, substitution_id: str = None, signer: Signer = None) -> CancelationAcknowledgment:
        raise NotImplementedError()

    def recover(self, document_id: str, accept: Accept = Accept.XML) -> Document:
        raise NotImplementedError()

    def rfc_valid(self, rfc: str | list[str]) -> bool | list[bool]:
        raise NotImplementedError()



import json
from datetime import datetime
from decimal import Decimal
from unittest import mock

from requests.auth import HTTPBasicAuth

from satcfdi.create.cfd import cfdi40
from satcfdi.pacs import Environment
from satcfdi.pacs.prodigia import Prodigia
from utils import get_signer, verify_result


def test_prodigia_test():
    prodigia = Prodigia(
        contrato="ccde5231-5bfe-4cba-b991-a59424336d4d",
        user="user",
        password="password",
        environment=Environment.TEST
    )

    signer = get_signer('xiqb891116qe4')
    emisor = cfdi40.Emisor(
        rfc=signer.rfc,
        nombre=signer.legal_name,
        regimen_fiscal="601"
    )

    invoice = cfdi40.Comprobante(
        emisor=emisor,
        lugar_expedicion="27200",
        fecha=datetime.fromisoformat("2022-09-28T22:40:38"),
        receptor=cfdi40.Receptor(
            rfc="H&E951128469",
            nombre="H & E",
            uso_cfdi="G03",
            domicilio_fiscal_receptor="34500",
            regimen_fiscal_receptor="601"
        ),
        metodo_pago="PPD",
        forma_pago="99",
        serie="T",
        folio="1000",
        conceptos=cfdi40.Concepto(
            cuenta_predial='1234567890',
            clave_prod_serv='10101702',
            cantidad=Decimal('.10'),
            clave_unidad='E48',
            descripcion='SERVICIOS DE RENTA',
            valor_unitario=Decimal("1.00"),
            impuestos=cfdi40.Impuestos(
                traslados=cfdi40.Impuesto.parse('002|Tasa|0.160000'),
                retenciones=[cfdi40.Impuesto.parse('001|Tasa|0.100000'), cfdi40.Impuesto.parse('002|Tasa|0.106667')],
            ),
            _traslados_incluidos=True
        )
    )
    invoice.sign(signer)

    with mock.patch(f'requests.request') as mk:
        mk.return_value.json = mock.Mock(return_value={
            'servicioTimbrado': {
                'codigo': 0,
                'selloSAT': 'dGhpcyBpcyBhIHNlYWwgdGVzdA==',
                'xmlBase64': 'PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiPz48Y2ZkaTpDb21wcm9iYW50ZSB4bWxuczpjZmRpPSJodHRwOi8vd3d3LnNhdC5nb2IubXgvY2ZkLzQiIA==',
                'contrato': '97353117-126d-4d72-93aa-1142496fbd81',
                'timbradoOk': True,
                'id': '21afd57c-20a3-4d48-a478-76de7b3fe4f4',
                'mensaje': '*** generado en modo de pruebas ***',
                'FechaTimbrado': '2020-01-01T112:20:00',
                'noCertificadoSAT': 12345678,
                'UUID': 'A6C30EF1-1967-4E4A-B4A0-EDAC0CF88541',
                'version': 1.1
            }
        })

        res = prodigia.stamp(
            cfdi=invoice
        )
        assert res.xml == b'<?xml version="1.0" encoding="UTF-8"?><cfdi:Comprobante xmlns:cfdi="http://www.sat.gob.mx/cfd/4" '
        assert not res.pdf

        assert mk.called
        mk.call_args.kwargs["headers"]["User-Agent"] = 'this is a test'
        assert mk.call_args.kwargs["auth"] == HTTPBasicAuth("user", "password")
        mk.call_args.kwargs["auth"] = "Basic abc"

        args = json.dumps(mk.call_args.kwargs, indent=2, default=str, ensure_ascii=False)
        verify = verify_result(data=args, filename="test_prodigia_stamp.json")
        assert verify



"""pfic http://www.sat.gob.mx/pfic"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class PFintegranteCoordinado(CFDI):
    """
    Este complemento permite incorporar a un Comprobante Fiscal Digital (CFD) o a un Comprobante Fiscal Digital a través de Internet (CFDI) los datos de identificación del vehículo que corresponda a personas físicas integrantes de coordinados, que opten por pagar el impuesto individualmente de conformidad con lo establecido por el artículo 83, séptimo párrafo de la Ley del Impuesto sobre la Renta.
    
    :param clave_vehicular: Atributo requerido para precisar Clave vehicular que corresponda a la versión del vehículo enajenado.
    :param placa: Atributo requerido para señalar la placa o número de folio del permiso del vehículo que corresponda.
    :param rfc_pf: Atributo opcional para precisar el RFC de la persona física integrante de coordinados, que opte por pagar el impuesto individualmente.
    """
    
    tag = '{http://www.sat.gob.mx/pfic}PFintegranteCoordinado'
    version = '1.0'
    
    def __init__(
            self,
            clave_vehicular: str,
            placa: str,
            rfc_pf: str = None,
    ): 
        super().__init__({
            'Version': self.version,
            'ClaveVehicular': clave_vehicular,
            'Placa': placa,
            'RFCPF': rfc_pf,
        })
        




"""cfdi http://www.sat.gob.mx/cfd/3"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class TInformacionAduanera(ScalarMap):
    """
    Tipo definido para expresar información aduanera
    
    :param numero: Atributo requerido para expresar el número del documento aduanero que ampara la importación del bien.
    :param fecha: Atributo requerido para expresar la fecha de expedición del documento aduanero que ampara la importación del bien. Se expresa en el formato aaaa-mm-dd
    :param aduana: Atributo opcional para precisar el nombre de la aduana por la que se efectuó la importación del bien.
    """
    
    def __init__(
            self,
            numero: str,
            fecha: date,
            aduana: str = None,
    ): 
        super().__init__({
            'Numero': numero,
            'Fecha': fecha,
            'Aduana': aduana,
        })
        

class TUbicacionFiscal(ScalarMap):
    """
    Tipo definido para expresar domicilios o direcciones
    
    :param calle: Este atributo requerido sirve para precisar la avenida, calle, camino o carretera donde se da la ubicación.
    :param municipio: Atributo requerido que sirve para precisar el municipio o delegación (en el caso del Distrito Federal) en donde se da la ubicación.
    :param estado: Atributo requerido que sirve para precisar el estado o entidad federativa donde se da la ubicación.
    :param pais: Atributo requerido que sirve para precisar el país donde se da la ubicación.
    :param codigo_postal: Atributo requerido que sirve para asentar el código postal en donde se da la ubicación.
    :param no_exterior: Este atributo opcional sirve para expresar el número particular en donde se da la ubicación sobre una calle dada.
    :param no_interior: Este atributo opcional sirve para expresar información adicional para especificar la ubicación cuando calle y número exterior (noExterior) no resulten suficientes para determinar la ubicación de forma precisa.
    :param colonia: Este atributo opcional sirve para precisar la colonia en donde se da la ubicación cuando se desea ser más específico en casos de ubicaciones urbanas.
    :param localidad: Atributo opcional que sirve para precisar la ciudad o población donde se da la ubicación.
    :param referencia: Atributo opcional para expresar una referencia de ubicación adicional.
    """
    
    def __init__(
            self,
            calle: str,
            municipio: str,
            estado: str,
            pais: str,
            codigo_postal: str,
            no_exterior: str = None,
            no_interior: str = None,
            colonia: str = None,
            localidad: str = None,
            referencia: str = None,
    ): 
        super().__init__({
            'Calle': calle,
            'Municipio': municipio,
            'Estado': estado,
            'Pais': pais,
            'CodigoPostal': codigo_postal,
            'NoExterior': no_exterior,
            'NoInterior': no_interior,
            'Colonia': colonia,
            'Localidad': localidad,
            'Referencia': referencia,
        })
        

class TUbicacion(ScalarMap):
    """
    Tipo definido para expresar domicilios o direcciones
    
    :param pais: Atributo requerido que sirve para precisar el país donde se da la ubicación.
    :param calle: Este atributo opcional sirve para precisar la avenida, calle, camino o carretera donde se da la ubicación.
    :param no_exterior: Este atributo opcional sirve para expresar el número particular en donde se da la ubicación sobre una calle dada.
    :param no_interior: Este atributo opcional sirve para expresar información adicional para especificar la ubicación cuando calle y número exterior (noExterior) no resulten suficientes para determinar la ubicación de forma precisa.
    :param colonia: Este atributo opcional sirve para precisar la colonia en donde se da la ubicación cuando se desea ser más específico en casos de ubicaciones urbanas.
    :param localidad: Atributo opcional que sirve para precisar la ciudad o población donde se da la ubicación.
    :param referencia: Atributo opcional para expresar una referencia de ubicación adicional.
    :param municipio: Atributo opcional que sirve para precisar el municipio o delegación (en el caso del Distrito Federal) en donde se da la ubicación.
    :param estado: Atributo opcional que sirve para precisar el estado o entidad federativa donde se da la ubicación.
    :param codigo_postal: Atributo opcional que sirve para asentar el código postal en donde se da la ubicación.
    """
    
    def __init__(
            self,
            pais: str,
            calle: str = None,
            no_exterior: str = None,
            no_interior: str = None,
            colonia: str = None,
            localidad: str = None,
            referencia: str = None,
            municipio: str = None,
            estado: str = None,
            codigo_postal: str = None,
    ): 
        super().__init__({
            'Pais': pais,
            'Calle': calle,
            'NoExterior': no_exterior,
            'NoInterior': no_interior,
            'Colonia': colonia,
            'Localidad': localidad,
            'Referencia': referencia,
            'Municipio': municipio,
            'Estado': estado,
            'CodigoPostal': codigo_postal,
        })
        

class Traslado(ScalarMap):
    """
    Nodo para la información detallada de un traslado de impuesto específico
    
    :param impuesto: Atributo requerido para señalar el tipo de impuesto trasladado
    :param tasa: Atributo requerido para señalar la tasa del impuesto que se traslada por cada concepto amparado en el comprobante
    :param importe: Atributo requerido para señalar el importe del impuesto trasladado
    """
    
    def __init__(
            self,
            impuesto: str,
            tasa: Decimal | int,
            importe: Decimal | int,
    ): 
        super().__init__({
            'Impuesto': impuesto,
            'Tasa': tasa,
            'Importe': importe,
        })
        

class Retencion(ScalarMap):
    """
    Nodo para la información detallada de una retención de impuesto específico
    
    :param impuesto: Atributo requerido para señalar el tipo de impuesto retenido
    :param importe: Atributo requerido para señalar el importe o monto del impuesto retenido
    """
    
    def __init__(
            self,
            impuesto: str,
            importe: Decimal | int,
    ): 
        super().__init__({
            'Impuesto': impuesto,
            'Importe': importe,
        })
        

class Impuestos(ScalarMap):
    """
    Nodo requerido para capturar los impuestos aplicables.
    
    :param total_impuestos_retenidos: Atributo opcional para expresar el total de los impuestos retenidos que se desprenden de los conceptos expresados en el comprobante fiscal digital a través de Internet.
    :param total_impuestos_trasladados: Atributo opcional para expresar el total de los impuestos trasladados que se desprenden de los conceptos expresados en el comprobante fiscal digital a través de Internet.
    :param retenciones: Nodo opcional para capturar los impuestos retenidos aplicables
    :param traslados: Nodo opcional para asentar o referir los impuestos trasladados aplicables
    """
    
    def __init__(
            self,
            total_impuestos_retenidos: Decimal | int = None,
            total_impuestos_trasladados: Decimal | int = None,
            retenciones: Retencion | dict | Sequence[Retencion | dict] = None,
            traslados: Traslado | dict | Sequence[Traslado | dict] = None,
    ): 
        super().__init__({
            'TotalImpuestosRetenidos': total_impuestos_retenidos,
            'TotalImpuestosTrasladados': total_impuestos_trasladados,
            'Retenciones': retenciones,
            'Traslados': traslados,
        })
        

class Parte(ScalarMap):
    """
    Nodo opcional para expresar las partes o componentes que integran la totalidad del concepto expresado en el comprobante fiscal digital a través de Internet
    
    :param cantidad: Atributo requerido para precisar la cantidad de bienes o servicios del tipo particular definido por la presente parte.
    :param descripcion: Atributo requerido para precisar la descripción del bien o servicio cubierto por la presente parte.
    :param unidad: Atributo opcional para precisar la unidad de medida aplicable para la cantidad expresada en la parte.
    :param no_identificacion: Atributo opcional para expresar el número de serie del bien o identificador del servicio amparado por la presente parte.
    :param valor_unitario: Atributo opcional para precisar el valor o precio unitario del bien o servicio cubierto por la presente parte.
    :param importe: Atributo opcional para precisar el importe total de los bienes o servicios de la presente parte. Debe ser equivalente al resultado de multiplicar la cantidad por el valor unitario expresado en la parte.
    :param informacion_aduanera: Nodo opcional para introducir la información aduanera aplicable cuando se trate de partes o componentes importados vendidos de primera mano.
    """
    
    def __init__(
            self,
            cantidad: Decimal | int,
            descripcion: str,
            unidad: str = None,
            no_identificacion: str = None,
            valor_unitario: Decimal | int = None,
            importe: Decimal | int = None,
            informacion_aduanera: TInformacionAduanera | dict | Sequence[TInformacionAduanera | dict] = None,
    ): 
        super().__init__({
            'Cantidad': cantidad,
            'Descripcion': descripcion,
            'Unidad': unidad,
            'NoIdentificacion': no_identificacion,
            'ValorUnitario': valor_unitario,
            'Importe': importe,
            'InformacionAduanera': informacion_aduanera,
        })
        

class Concepto(ScalarMap):
    """
    Nodo para introducir la información detallada de un bien o servicio amparado en el comprobante.
    
    :param cantidad: Atributo requerido para precisar la cantidad de bienes o servicios del tipo particular definido por el presente concepto.
    :param unidad: Atributo requerido para precisar la unidad de medida aplicable para la cantidad expresada en el concepto.
    :param descripcion: Atributo requerido para precisar la descripción del bien o servicio cubierto por el presente concepto.
    :param valor_unitario: Atributo requerido para precisar el valor o precio unitario del bien o servicio cubierto por el presente concepto.
    :param importe: Atributo requerido para precisar el importe total de los bienes o servicios del presente concepto. Debe ser equivalente al resultado de multiplicar la cantidad por el valor unitario expresado en el concepto.
    :param no_identificacion: Atributo opcional para expresar el número de serie del bien o identificador del servicio amparado por el presente concepto.
    :param informacion_aduanera: Nodo opcional para introducir la información aduanera aplicable cuando se trate de ventas de primera mano de mercancías importadas.
    :param cuenta_predial: Nodo opcional para asentar el número de cuenta predial con el que fue registrado el inmueble, en el sistema catastral de la entidad federativa de que trate, o bien para incorporar los datos de identificación del certificado de participación inmobiliaria no amortizable.
    :param complemento_concepto: Nodo opcional donde se incluirán los nodos complementarios de extensión al concepto, definidos por el SAT, de acuerdo a disposiciones particulares a un sector o actividad especifica.
    :param parte: Nodo opcional para expresar las partes o componentes que integran la totalidad del concepto expresado en el comprobante fiscal digital a través de Internet
    """
    
    def __init__(
            self,
            cantidad: Decimal | int,
            unidad: str,
            descripcion: str,
            valor_unitario: Decimal | int,
            importe: Decimal | int,
            no_identificacion: str = None,
            informacion_aduanera: TInformacionAduanera | dict | Sequence[TInformacionAduanera | dict] = None,
            cuenta_predial: str = None,
            complemento_concepto: XElement | Sequence[XElement] = None,
            parte: Parte | dict | Sequence[Parte | dict] = None,
    ): 
        super().__init__({
            'Cantidad': cantidad,
            'Unidad': unidad,
            'Descripcion': descripcion,
            'ValorUnitario': valor_unitario,
            'Importe': importe,
            'NoIdentificacion': no_identificacion,
            'InformacionAduanera': informacion_aduanera,
            'CuentaPredial': cuenta_predial,
            'ComplementoConcepto': complemento_concepto,
            'Parte': parte,
        })
        

class Receptor(ScalarMap):
    """
    Nodo requerido para precisar la información del contribuyente receptor del comprobante.
    
    :param rfc: Atributo requerido para precisar la Clave del Registro Federal de Contribuyentes correspondiente al contribuyente receptor del comprobante.
    :param nombre: Atributo opcional para el nombre, denominación o razón social del contribuyente receptor del comprobante.
    :param domicilio: Nodo opcional para la definición de la ubicación donde se da el domicilio del receptor del comprobante fiscal.
    """
    
    def __init__(
            self,
            rfc: str,
            nombre: str = None,
            domicilio: TUbicacion | dict = None,
    ): 
        super().__init__({
            'Rfc': rfc,
            'Nombre': nombre,
            'Domicilio': domicilio,
        })
        

class Emisor(ScalarMap):
    """
    Nodo requerido para expresar la información del contribuyente emisor del comprobante.
    
    :param rfc: Atributo requerido para la Clave del Registro Federal de Contribuyentes correspondiente al contribuyente emisor del comprobante sin guiones o espacios.
    :param regimen_fiscal: Nodo requerido para incorporar los regímenes en los que tributa el contribuyente emisor. Puede contener más de un régimen.
    :param nombre: Atributo opcional para el nombre, denominación o razón social del contribuyente emisor del comprobante.
    :param domicilio_fiscal: Nodo opcional para precisar la información de ubicación del domicilio fiscal del contribuyente emisor
    :param expedido_en: Nodo opcional para precisar la información de ubicación del domicilio en donde es emitido el comprobante fiscal en caso de que sea distinto del domicilio fiscal del contribuyente emisor.
    """
    
    def __init__(
            self,
            rfc: str,
            regimen_fiscal: str | Sequence[str],
            nombre: str = None,
            domicilio_fiscal: TUbicacionFiscal | dict = None,
            expedido_en: TUbicacion | dict = None,
    ): 
        super().__init__({
            'Rfc': rfc,
            'RegimenFiscal': regimen_fiscal,
            'Nombre': nombre,
            'DomicilioFiscal': domicilio_fiscal,
            'ExpedidoEn': expedido_en,
        })
        

class Comprobante(CFDI):
    """
    Estándar de Comprobante fiscal digital a través de Internet.
    
    :param fecha: Atributo requerido para la expresión de la fecha y hora de expedición del comprobante fiscal. Se expresa en la forma aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO 8601.
    :param sello: Atributo requerido para contener el sello digital del comprobante fiscal, al que hacen referencia las reglas de resolución miscelánea aplicable. El sello deberá ser expresado cómo una cadena de texto en formato Base 64.
    :param forma_de_pago: Atributo requerido para precisar la forma de pago que aplica para este comprobnante fiscal digital a través de Internet. Se utiliza para expresar Pago en una sola exhibición o número de parcialidad pagada contra el total de parcialidades, Parcialidad 1 de X.
    :param no_certificado: Atributo requerido para expresar el número de serie del certificado de sello digital que ampara al comprobante, de acuerdo al acuse correspondiente a 20 posiciones otorgado por el sistema del SAT.
    :param certificado: Atributo requerido que sirve para expresar el certificado de sello digital que ampara al comprobante como texto, en formato base 64.
    :param sub_total: Atributo requerido para representar la suma de los importes antes de descuentos e impuestos.
    :param total: Atributo requerido para representar la suma del subtotal, menos los descuentos aplicables, más los impuestos trasladados, menos los impuestos retenidos.
    :param tipo_de_comprobante: Atributo requerido para expresar el efecto del comprobante fiscal para el contribuyente emisor.
    :param metodo_de_pago: Atributo requerido de texto libre para expresar el método de pago de los bienes o servicios amparados por el comprobante. Se entiende como método de pago leyendas tales como: cheque, tarjeta de crédito o debito, depósito en cuenta, etc.
    :param lugar_expedicion: Atributo requerido para incorporar el lugar de expedición del comprobante.
    :param emisor: Nodo requerido para expresar la información del contribuyente emisor del comprobante.
    :param receptor: Nodo requerido para precisar la información del contribuyente receptor del comprobante.
    :param conceptos: Nodo requerido para enlistar los conceptos cubiertos por el comprobante.
    :param impuestos: Nodo requerido para capturar los impuestos aplicables.
    :param serie: Atributo opcional para precisar la serie para control interno del contribuyente. Este atributo acepta una cadena de caracteres alfabéticos de 1 a 25 caracteres sin incluir caracteres acentuados.
    :param folio: Atributo opcional para control interno del contribuyente que acepta un valor numérico entero superior a 0 que expresa el folio del comprobante.
    :param condiciones_de_pago: Atributo opcional para expresar las condiciones comerciales aplicables para el pago del comprobante fiscal digital a través de Internet.
    :param descuento: Atributo opcional para representar el importe total de los descuentos aplicables antes de impuestos.
    :param motivo_descuento: Atributo opcional para expresar el motivo del descuento aplicable.
    :param tipo_cambio: Atributo opcional para representar el tipo de cambio conforme a la moneda usada
    :param moneda: Atributo opcional para expresar la moneda utilizada para expresar los montos
    :param num_cta_pago: Atributo Opcional para incorporar al menos los cuatro últimos digitos del número de cuenta con la que se realizó el pago.
    :param folio_fiscal_orig: Atributo opcional para señalar el número de folio fiscal del comprobante que se hubiese expedido por el valor total del comprobante, tratándose del pago en parcialidades.
    :param serie_folio_fiscal_orig: Atributo opcional para señalar la serie del folio del comprobante que se hubiese expedido por el valor total del comprobante, tratándose del pago en parcialidades.
    :param fecha_folio_fiscal_orig: Atributo opcional para señalar la fecha de expedición del comprobante que se hubiese emitido por el valor total del comprobante, tratándose del pago en parcialidades. Se expresa en la forma aaaa-mm-ddThh:mm:ss, de acuerdo con la especificación ISO 8601.
    :param monto_folio_fiscal_orig: Atributo opcional para señalar el total del comprobante que se hubiese expedido por el valor total de la operación, tratándose del pago en parcialidades
    :param complemento: Nodo opcional donde se incluirá el complemento Timbre Fiscal Digital de manera obligatoria y los nodos complementarios determinados por el SAT, de acuerdo a las disposiciones particulares a un sector o actividad específica.
    :param addenda: Nodo opcional para recibir las extensiones al presente formato que sean de utilidad al contribuyente. Para las reglas de uso del mismo, referirse al formato de origen.
    """
    
    tag = '{http://www.sat.gob.mx/cfd/3}Comprobante'
    version = '3.2'
    
    def __init__(
            self,
            fecha: datetime,
            sello: str,
            forma_de_pago: str,
            no_certificado: str,
            certificado: str,
            sub_total: Decimal | int,
            total: Decimal | int,
            tipo_de_comprobante: str,
            metodo_de_pago: str,
            lugar_expedicion: str,
            emisor: Emisor | dict,
            receptor: Receptor | dict,
            conceptos: Concepto | dict | Sequence[Concepto | dict],
            impuestos: Impuestos | dict,
            serie: str = None,
            folio: str = None,
            condiciones_de_pago: str = None,
            descuento: Decimal | int = None,
            motivo_descuento: str = None,
            tipo_cambio: str = None,
            moneda: str = None,
            num_cta_pago: str = None,
            folio_fiscal_orig: str = None,
            serie_folio_fiscal_orig: str = None,
            fecha_folio_fiscal_orig: datetime = None,
            monto_folio_fiscal_orig: Decimal | int = None,
            complemento: XElement | Sequence[XElement] = None,
            addenda: XElement | Sequence[XElement] = None,
    ): 
        super().__init__({
            'Version': self.version,
            'Fecha': fecha,
            'Sello': sello,
            'FormaDePago': forma_de_pago,
            'NoCertificado': no_certificado,
            'Certificado': certificado,
            'SubTotal': sub_total,
            'Total': total,
            'TipoDeComprobante': tipo_de_comprobante,
            'MetodoDePago': metodo_de_pago,
            'LugarExpedicion': lugar_expedicion,
            'Emisor': emisor,
            'Receptor': receptor,
            'Conceptos': conceptos,
            'Impuestos': impuestos,
            'Serie': serie,
            'Folio': folio,
            'CondicionesDePago': condiciones_de_pago,
            'Descuento': descuento,
            'MotivoDescuento': motivo_descuento,
            'TipoCambio': tipo_cambio,
            'Moneda': moneda,
            'NumCtaPago': num_cta_pago,
            'FolioFiscalOrig': folio_fiscal_orig,
            'SerieFolioFiscalOrig': serie_folio_fiscal_orig,
            'FechaFolioFiscalOrig': fecha_folio_fiscal_orig,
            'MontoFolioFiscalOrig': monto_folio_fiscal_orig,
            'Complemento': complemento,
            'Addenda': addenda,
        })
        




import os

from satcfdi.portal.utils import generate_token, verify_token, action_url
from satcfdi.portal import SATPortal
from tests.utils import get_signer

current_dir = os.path.dirname(__file__)
test_dir = os.path.join(current_dir, 'test_portal')
session_file = os.path.join(test_dir, 'session')


def test_portal_get_token():
    code = 'MjFhYzNkODAtN2MxMC00NGU5LWI5NDYtMThmZWZmMjM5OTQ1'
    signer = get_signer('cacx7605101p8')
    token = generate_token(signer, code=code)

    assert code == verify_token(certificate=signer, token=token)


def test_action_url():
    target_url = action_url(
        action="/nidp/app/login?id=fiel&sid=0&option=credential&sid=0",
        url='https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel'
    )
    assert target_url == 'https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel&sid=0&option=credential&sid=0'

    target_url = action_url(
        action=None,
        url='https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel'
    )
    assert target_url == 'https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel'

    target_url = action_url(
        action="http://aaaa.bbb",
        url='https://loginda.siat.sat.gob.mx/nidp/app/login?id=fiel'
    )
    assert target_url == "http://aaaa.bbb"


def test_start_session():
    signer = get_signer('cacx7605101p8')
    res = SATPortal(signer)

    try:
        with open(session_file, 'rb') as f:
            res.load_session(f)
        print('Session Loaded')
    except FileNotFoundError:
        print('FileNotFound')

    # res.login()

    os.makedirs(test_dir, exist_ok=True)
    with open(session_file, 'wb') as f:
        res.save_session(f)



"""ventavehiculos http://www.sat.gob.mx/ventavehiculos"""
from decimal import Decimal
from datetime import datetime, date, time
from collections.abc import Sequence
from ... import CFDI, XElement, ScalarMap


class TInformacionAduanera(ScalarMap):
    """
    Tipo definido para expresar información aduanera
    
    :param numero: Atributo requerido para expresar el número del documento aduanero que ampara la importación del bien.
    :param fecha: Atributo requerido para expresar la fecha de expedición del documento aduanero que ampara la importación del bien.
    :param aduana: Atributo opcional para precisar la aduana por la que se efectuó la importación del bien.
    """
    
    def __init__(
            self,
            numero: str,
            fecha: date,
            aduana: str = None,
    ): 
        super().__init__({
            'Numero': numero,
            'Fecha': fecha,
            'Aduana': aduana,
        })
        

class Parte(ScalarMap):
    """
    Nodo opcional para expresar las partes o componentes que integran la totalidad del concepto expresado en el CFDI.
    
    :param cantidad: Atributo requerido para precisar la cantidad de bienes o servicios del tipo particular definido por la presente parte.
    :param descripcion: Atributo requerido para precisar la descripción del bien o servicio cubierto por la presente parte.
    :param unidad: Atributo opcional para precisar la unidad de medida aplicable para la cantidad expresada en la parte.
    :param no_identificacion: Atributo opcional para expresar el número de serie del bien o identificador del servicio amparado por la presente parte.
    :param valor_unitario: Atributo opcional para precisar el valor o precio unitario del bien o servicio cubierto por la presente parte.
    :param importe: Atributo opcional para precisar el importe total de los bienes o servicios de la presente parte. Debe ser equivalente al resultado de multiplicar la cantidad por el valor unitario expresado en la parte.
    :param informacion_aduanera: Nodo opcional para introducir la información aduanera aplicable cuando se trate de partes o componentes importados vendidos de primera mano.
    """
    
    def __init__(
            self,
            cantidad: Decimal | int,
            descripcion: str,
            unidad: str = None,
            no_identificacion: str = None,
            valor_unitario: Decimal | int = None,
            importe: Decimal | int = None,
            informacion_aduanera: TInformacionAduanera | dict | Sequence[TInformacionAduanera | dict] = None,
    ): 
        super().__init__({
            'Cantidad': cantidad,
            'Descripcion': descripcion,
            'Unidad': unidad,
            'NoIdentificacion': no_identificacion,
            'ValorUnitario': valor_unitario,
            'Importe': importe,
            'InformacionAduanera': informacion_aduanera,
        })
        

class VentaVehiculos(CFDI):
    """
    Complemento concepto que permite incorporar a los fabricantes, ensambladores o distribuidores autorizados de automóviles nuevos, así como aquéllos que importen automóviles para permanecer en forma definitiva en la franja fronteriza norte del país y en los Estados de Baja California, Baja California Sur y la región parcial del Estado de Sonora, a un Comprobante Fiscal Digital a través de Internet (CFDI) la clave vehicular que corresponda a la versión enajenada y el número de identificación vehicular que corresponda al vehículo enajenado.
    
    :param clave_vehicular: Atributo requerido para precisar Clave vehicular que corresponda a la versión del vehículo enajenado.
    :param niv: Atributo requerido para precisar el número de identificación vehicular que corresponda al vehículo enajenado.
    :param informacion_aduanera: Nodo opcional para introducir la información aduanera aplicable cuando se trate de ventas de primera mano de mercancías importadas.
    :param parte: Nodo opcional para expresar las partes o componentes que integran la totalidad del concepto expresado en el CFDI.
    """
    
    tag = '{http://www.sat.gob.mx/ventavehiculos}VentaVehiculos'
    version = '1.1'
    
    def __init__(
            self,
            clave_vehicular: str,
            niv: str,
            informacion_aduanera: TInformacionAduanera | dict | Sequence[TInformacionAduanera | dict] = None,
            parte: Parte | dict | Sequence[Parte | dict] = None,
    ): 
        super().__init__({
            'Version': self.version,
            'ClaveVehicular': clave_vehicular,
            'Niv': niv,
            'InformacionAduanera': informacion_aduanera,
            'Parte': parte,
        })
        

